#!/usr/bin/python
# Filename: dieroller.py
# Author: Tim Doty
# Version: v0.4.0 20080803
# Distribution: Public Domain

import wx
import math
import random

ID_ABOUT   = 101
ID_OPEN    = 102
ID_BUTTON1 = 110
ID_EXIT    = 110

class RowSet(object):
    """ Provides dice and row handling """
    def __init__(self, panel, numRows=1):
        self.maxRows = 25
        self.panel = panel
        self.window = panel.GetTopLevelParent()
        self.sizer = panel.GetSizer()
        #
        # possible operators. to use them would need to do a preprocessing to translate
        # into single char occurances
        #  log, ln
        #  sin, cos, tan, asin, acos, atan
        #  round, int, floor, ceil
        #  ?:
        #  adjSTR (results in die roll for indicated adjSTR)
        ##
        # a : adj (1->1, 2->1, 3->2, etc.)
        # p : points for adj (1->1, 2->3, 3->6, etc.)
        self.maxPrecedence = 99
        self.operatorPrecedence = '(aApds^*/+-)'
        self.operatorUnary = '(aAp'
        self.scaled = {}
        self.scaled[6] = [1, 1, 1, 2, 2, 3]
        self.scaled[8] = [1, 1, 1, 2, 2, 2, 3, 3]
        self.scaled[10] = [1, 1, 1, 1, 2, 2, 2, 3, 3, 4]
        self.scaled[12] = [1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 4, 4]
        self.scaled[20] = [1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4, 5, 5, 6]
        #
    def Tokenize(self, diceToRoll):
        diceToRoll = diceToRoll.replace('adjSTR', 'A')
        diceToRoll = diceToRoll.replace('adj', 'a')
        diceToRoll = diceToRoll.replace('points', 'p')
        diceToRoll = diceToRoll.replace('point', 'p')
        diceToRoll = diceToRoll.replace('pts', 'p')
        diceToRoll = diceToRoll.replace('pt', 'p')
        if diceToRoll == '':
            return [0]
        # tokenize the string
        parts = []
        appending = False
        for char in diceToRoll:
            if char in self.operatorPrecedence:
                parts += [char]
                appending = False
            else:
                if appending:
                   parts[-1] += char
                else:
                    parts += [char]
                    appending = True
        return parts
    def Evaluate(self, parts):
        self.stack = []
        self.lastOperator = self.maxPrecedence
        for token in parts:
            try:
                self.stack.append(float(token))
                a = self.stack.pop()
                if a == int(a):
                    self.stack.append(int(a))
                else:
                    self.stack.append(a)
            except ValueError:
                currentPrecedence = self.operatorPrecedence.find(token)
                while self.lastOperator != 0 and self.lastOperator < currentPrecedence:
                    # lower precedence, let's do it!
                    self.__DoStackOperation()
                if token == ')':
                    if self.lastOperator == 0:
                        self.__DoStackOperation()
                    else:
                        # hitting this when using nested parenthesis, something is wrong.
                        # however, as it is does not appear to impact correct results
                        # I'm disabling this until I can do a more thorough review
                        pass
                        #print "ERROR: incomplete operations within parens"
                else:
                    self.stack.append(token)
                self.lastOperator = currentPrecedence
        # after going through all tokens make sure to collapse stack
        while len(self.stack) > 1:
            self.__DoStackOperation()
        return self.stack.pop()
    def DoRoll(self, event):
        # read the dice to roll, for each roll, and put result in the result
        # for now ignoring number in set
        # parse and validate dice
        # consider using MiniParser http://christophe.delord.free.fr/mp/index.html
        diceToRoll = self.inputDice.GetLineText(0) # always 0 (line number to return)
        parts = self.Tokenize(diceToRoll)
        # now interpret the results
        for result in self.results:
            dieRoll = self.Evaluate(parts)
            result.ChangeValue(str(dieRoll))
    def __DoStackOperation(self):
        # removes top three items on stack, performs operation, puts result back onto stack
        operandRight = self.stack.pop()
        operator = self.stack.pop()
        if self.operatorUnary.find(operator) != -1:
            # which unary operation?
            if operator == '(':
                self.stack.append(operandRight)
            elif operator == 'a':
                # adj (scaled number)
                self.stack.append( int((math.sqrt(8 * operandRight + 1) - 1) / 2) )
            elif operator == 'p':
                # inverse of scaled number
                self.stack.append( (operandRight * operandRight + operandRight) / 2 )
            elif operator == 'A':
                # adjSTR
                operrolls = {
                          0.10: [],
                          0.50: [(0,  1)],
                          1.06: [(0,  2)],
                          1.75: [(0,  3)],
                          2.56: [(1,  3)],
                          3.93: [(1,  4)],
                          6.05: [(1,  6)],
                          8.45: [(1,  8)],
                         11.11: [(1, 10)],
                         13.26: [(1, 12)],
                         15.53: [(1,  6), (1,  6)],
                         18.73: [(1,  6), (1,  8)],
                         22.13: [(1,  8), (1,  8)],
                         25.72: [(1,  8), (1, 10)],
                         29.48: [(1, 10), (1, 10)],
                         33.40: [(1, 10), (1, 12)],
                         36.45: [(1, 12), (1, 12)],
                         39.59: [(1,  8), (1,  8)],
                         43.91: [(1,  8), (1,  8), (1, 10)],
                         48.37: [(1,  8), (1, 10), (1, 10)],
                         52.97: [(1, 10), (1, 10), (1, 10)],
                         57.72: [(1, 10), (1, 10), (1, 12)],
                         62.59: [(1, 10), (1, 12), (1, 12)],
                         66.33: [(1, 12), (1, 12), (1, 12)],
                         70.15: [(1,  8), (1,  8), (1, 10), (1, 10)],
                         75.34: [(1,  8), (1, 10), (1, 10), (1, 10)],
                         80.66: [(1, 10), (1, 10), (1, 10), (1, 10)],
                         86.09: [(1, 10), (1, 10), (1, 10), (1, 12)],
                         91.65: [(1, 10), (1, 10), (1, 12), (1, 12)],
                         97.32: [(1, 10), (1, 12), (1, 12), (1, 12)],
                        101.64: [(1, 12), (1, 12), (1, 12), (1, 12)],
                        106.03: [(1,  8), (1, 10), (1, 10), (1, 10), (1, 10)],
                        111.97: [(1, 10), (1, 10), (1, 10), (1, 10), (1, 10)],
                        118.02: [(1, 10), (1, 10), (1, 10), (1, 10), (1, 12)],
                        124.18: [(1, 10), (1, 10), (1, 10), (1, 12), (1, 12)],
                        130.44: [(1, 10), (1, 10), (1, 12), (1, 12), (1, 12)],
                        136.80: [(1, 10), (1, 12), (1, 12), (1, 12), (1, 12)]
                        }
                rolled = False
                for i in sorted(operrolls.keys()):
                    if operandRight < i:
                        rolled = True
                        roll = 0
                        for die in operrolls[i]:
                            if 0 < die[1]:
                                roll += random.randint(die[0], die[1])
                        self.stack.append(roll)
                        break;
                if not rolled:
                    # the really high adjSTR
                    avgDmg = math.pow(operandRight / .7557, 2.0/3)
                    numD12 = int(avgDmg / 6.5 + 0.5)
                    roll = 0
                    while numD12 > 0:
                        roll += random.randint(1, 12)
                        numD12 -= 1
                    self.stack.append(roll)
            else:
                print "ERROR: unknown unary operator '", operator, "'"
        else:
            # get first operand
            operandLeft = self.stack.pop()
            # which binary operation?
            if operator == 'd':
                roll = 0
                if operandRight > 0:
                    while operandLeft > 0:
                        roll += random.randint(1, int(operandRight))
                        operandLeft -= 1
                self.stack.append(roll)
            elif operator == 's':
                roll = 0
                while operandLeft > 0:
                    result = random.randint(1, operandRight)
                    try:
                        roll += self.scaled[operandRight][result-1]
                    except KeyError:
                        print "ERROR: not a valid scaled die size '", operandRight, "'"
                    operandLeft -= 1
                self.stack.append(roll)
            elif operator == '+':
                self.stack.append(operandLeft + operandRight)
            elif operator == '-':
                self.stack.append(operandLeft - operandRight)
            elif operator == '*':
                self.stack.append(operandLeft * operandRight)
            elif operator == '/':
                self.stack.append(operandLeft / operandRight)
            elif operator == '^':
                self.stack.append(int(math.pow(operandLeft, operandRight)))
            else:
                print "ERROR: unknown binary operator '", operator, "'"
                self.stack.append(operandLeft)
        # update "previous" operator precedence
        if len(self.stack) > 1:
            self.lastOperator = self.operatorPrecedence.find(self.stack[-2])
        else:
            self.lastOperator = self.maxPrecedence

class DiceSet(RowSet):
    """ Provides a set of arbitrary dice """
    def __init__(self, panel, indexBase=3, numRows=1):
        RowSet.__init__(self, panel=panel, numRows=numRows)
        self.results = [] # list of widgets for result output
        self.inputRows = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER)
        self.inputDice = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER)
        self.sizerResult = wx.BoxSizer(wx.VERTICAL)
        self.inputRows.ChangeValue(str(numRows))
        self.sizer.Insert(indexBase + 0, self.inputRows)
        self.sizer.Insert(indexBase + 1, self.inputDice)
        self.sizer.Insert(indexBase + 2, self.sizerResult)
        self.inputRows.Bind(wx.EVT_TEXT_ENTER, self.UpdateSet)
        self.inputDice.Bind(wx.EVT_TEXT_ENTER, self.UpdateSet)
        for row in range(0, numRows):
            result = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER | wx.TE_READONLY)
            self.results.append(result)
            self.sizerResult.Add(result)
    def Destroy(self):
        for row in range(0, len(self.results)):
            self.sizerResult.Remove(row)
            item = self.results.pop()
            item.Destroy()
        self.sizer.Remove(self.inputRows)
        self.sizer.Remove(self.inputDice)
        self.sizer.Remove(self.sizerResult)
        self.inputRows.Destroy()
        self.inputDice.Destroy()
        # need to trigger a re-layout
        self.window.Layout()
    def UpdateSet(self, event):
        try:
            numRows = int(self.inputRows.GetLineText(0)) # always 0 (line number to return)
            if numRows < 1:
                numRows = 1
            elif numRows > self.maxRows: # sanity check to avoid huge sizes
                numRows = self.maxRows
            self.inputRows.ChangeValue(str(numRows))
        except ValueError:
            numRows = len(self.results)
            self.inputRows.ChangeValue(numRows)
        if numRows < len(self.results):
            # need to remove some result rows
            # removes from sizer and result stack in opposite orders, but since it
            # is the same number of items and the removals are independent this is
            # not an issue
            for row in range( numRows, len(self.results) ):
                self.sizerResult.Remove(row)
                item = self.results.pop()
                item.Destroy()
            # need to trigger a re-layout
            self.window.Layout()
        elif numRows > len(self.results):
            # need to add some result rows
            for row in range( len(self.results), numRows ):
                result = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER | wx.TE_READONLY)
                self.results.append(result)
                self.sizerResult.Add(result)
            # probably need to trigger a re-layout?
            self.window.Layout()
        self.DoRoll(event)

class SkillSet(RowSet):
    """ Provides handling for skill evaluation """
    def __init__(self, panel, indexBase=7, numRows=1):
        RowSet.__init__(self, panel=panel, numRows=numRows)
        self.rolls = [] # list of widgets for roll output
        self.results = [] # list of widgets for result output
        self.inputRows = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER)
        self.inputScore = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER)
        self.inputDifficulty = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER)
        self.inputAdj = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER)
        self.inputChance = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER | wx.TE_READONLY)
        self.sizerRoll = wx.BoxSizer(wx.VERTICAL)
        self.sizerResult = wx.BoxSizer(wx.VERTICAL)
        self.inputRows.ChangeValue(str(numRows))
        self.sizer.Insert(indexBase + 0, self.inputRows)
        self.sizer.Insert(indexBase + 1, self.inputScore)
        self.sizer.Insert(indexBase + 2, self.inputDifficulty)
        self.sizer.Insert(indexBase + 3, self.inputAdj)
        self.sizer.Insert(indexBase + 4, self.inputChance)
        self.sizer.Insert(indexBase + 5, self.sizerRoll)
        self.sizer.Insert(indexBase + 6, self.sizerResult)
        self.inputRows.Bind(wx.EVT_TEXT_ENTER, self.UpdateSet)
        self.inputScore.Bind(wx.EVT_TEXT_ENTER, self.UpdateSet)
        self.inputDifficulty.Bind(wx.EVT_TEXT_ENTER, self.UpdateSet)
        self.inputAdj.Bind(wx.EVT_TEXT_ENTER, self.UpdateSet)
        self.inputChance.Bind(wx.EVT_TEXT_ENTER, self.UpdateSet)
        for row in range(0, numRows):
            roll = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER)
            roll.Bind(wx.EVT_TEXT_ENTER, self.UpdateResult)
            self.rolls.append(roll)
            self.sizerRoll.Add(roll)
            result = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER | wx.TE_READONLY)
            self.results.append(result)
            self.sizerResult.Add(result)
        # Level of Success Table
        self.Critical3 = [ 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,2,3,5,8 ]
        self.Critical2 = [ 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,2,
                2,2,2,2,2,3,3,3,4,4,5,6,6,7,9,11,13,17,25,34 ]
        self.Critical = [ 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,3,
                3,3,3,3,3,4,4,4,4,5,5,5,5,6,6,6,7,7,7,8,8,9,9,10,10,11,12,
                12,13,14,15,16,17,18,19,20,22,24,25,28,30,33,36,40,45,52,
                63,72 ]
        self.Special = [ 0,0,0,0,0,0,1,1,1,1,1,1,1,2,2,2,2,3,3,3,3,4,4,4,
                4,5,5,5,6,6,6,7,7,8,8,8,9,9,10,10,11,11,11,12,12,13,14,14,
                15,15,16,16,17,18,18,19,20,20,21,22,23,24,24,25,26,27,28,
                29,30,31,32,33,34,35,36,37,38,40,41,42,44,45,47,48,50,51,
                53,55,57,59,61,63,66,68,71,74,77,81,85,91,94 ]
        self.Miss = [ 6,9,15,19,23,26,29,32,34,37,39,41,43,45,47,49,50,52,
                53,55,56,58,59,60,62,63,64,65,66,67,68,69,70,71,72,73,74,
                75,76,76,77,78,79,80,80,81,82,82,83,84,84,85,85,86,86,87,
                88,88,89,89,89,90,90,91,91,92,92,92,93,93,94,94,94,95,95,
                95,96,96,96,96,97,97,97,97,98,98,98,98,99,99,99,99,99,99,
                99,100,100,100,100,100,101 ]
        self.Failure = [ 28,37,48,55,60,64,67,70,72,75,76,78,70,81,82,83,
                84,85,86,87,88,88,89,90,90,91,91,92,92,93,93,93,94,94,94,
                95,95,95,95,96,96,96,96,97,97,97,97,97,97,98,98,98,98,98,
                98,98,98,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,100,
                100,100,100,100,100,100,100,100,100,100,100,100,100,100,
                100,100,100,100,100,100,100,100,101,101,101,101,101,101 ]
        self.Fumble = [ 66,75,83,87,89,91,93,94,94,95,96,96,97,97,97,98,
                98,98,98,98,98,99,99,99,99,99,99,99,99,99,99,99,99,99,100,
                100,100,100,100,100,100,100,100,100,100,100,100,100,100,
                100,100,100,100,100,100,100,100,100,100,100,100,100,100,
                100,100,100,100,100,100,100,100,100,101,101,101,101,101,
                101,101,101,101,101,101,101,101,101,101,101,101,101,101,
                101,101,101,101,101,101,101,101,101,101 ]
        self.Fumble2 = [ 92,95,97,98,99,99,99,99,100,100,100,100,100,100,
                100,100,100,100,100,100,100,100,100,100,100,100,100,100,
                100,100,100,100,100,100,101,101,101,101,101,101,101,101,
                101,101,101,101,101,101,101,101,101,101,101,101,101,101,
                101,101,101,101,101,101,101,101,101,101,101,101,101,101,
                101,101,101,101,101,101,101,101,101,101,101,101,101,101,
                101,101,101,101,101,101,101,101,101,101,101,101,101,101,
                101,101,101 ]
    def Destroy(self):
        for row in range(0, len(self.results)):
            self.sizerRoll.Remove(row)
            item = self.rolls.pop()
            item.Destroy()
            self.sizerResult.Remove(row)
            item = self.results.pop()
            item.Destroy()
        self.sizer.Remove(self.inputRows)
        self.sizer.Remove(self.inputScore)
        self.sizer.Remove(self.inputDifficulty)
        self.sizer.Remove(self.inputAdj)
        self.sizer.Remove(self.inputChance)
        self.sizer.Remove(self.sizerRoll)
        self.sizer.Remove(self.sizerResult)
        self.inputRows.Destroy()
        self.inputScore.Destroy()
        self.inputDifficulty.Destroy()
        self.inputAdj.Destroy()
        self.inputChance.Destroy()
        # need to trigger a re-layout
        self.window.Layout()
    def UpdateSet(self, event):
        try:
            numRows = int(self.inputRows.GetLineText(0)) # always 0 (line number to return)
            if numRows < 1:
                numRows = 1
            elif numRows > self.maxRows: # sanity check to avoid huge sizes
                numRows = self.maxRows
            self.inputRows.ChangeValue(str(numRows))
        except ValueError:
            numRows = len(self.results)
            self.inputRows.ChangeValue(numRows)
        if numRows < len(self.results):
            # need to remove some result rows
            # removes from sizer and result stack in opposite orders, but since it
            # is the same number of items and the removals are independent this is
            # not an issue
            for row in range( numRows, len(self.results) ):
                self.sizerRoll.Remove(row)
                self.sizerResult.Remove(row)
                item = self.rolls.pop()
                item.Destroy()
                item = self.results.pop()
                item.Destroy()
            # need to trigger a re-layout
            self.window.Layout()
        elif numRows > len(self.results):
            # need to add some result rows
            for row in range( len(self.results), numRows ):
                roll = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER)
                self.rolls.append(roll)
                self.sizerRoll.Add(roll)
                result = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER | wx.TE_READONLY)
                self.results.append(result)
                self.sizerResult.Add(result)
            # probably need to trigger a re-layout?
            self.window.Layout()
        self.DoSkillRoll(event)
    def UpdateResult(self, event):
        chance = int(self.inputChance.GetLineText(0))
        roll = int(self.Evaluate( self.Tokenize( self.rolls[0].GetLineText(0) ) ))
        self.rolls[0].ChangeValue(str(roll))
        result = self.GetResult(chance, roll)
        self.results[0].ChangeValue(result)
    def DoSkillRoll(self, event):
        score = float(self.Evaluate( self.Tokenize( self.inputScore.GetLineText(0) ) ))
        self.inputScore.ChangeValue(str(int(score)))
        difficulty = float(self.Evaluate( self.Tokenize( self.inputDifficulty.GetLineText(0) ) ))
        self.inputDifficulty.ChangeValue(str(int(difficulty)))
        adj = float(self.Evaluate( self.Tokenize( self.inputAdj.GetLineText(0) ) ))
        self.inputAdj.ChangeValue(str(int(adj)))
        if score == 0:
            chance = 0
        elif difficulty == 0:
            chance = 100
        elif score > difficulty:
            chance = 100 - int((difficulty * difficulty) / (score * score) * 50 + .5) + int(adj)
        elif score < difficulty:
            chance = int((score * score) / (difficulty * difficulty) * 50 + .5) + int(adj)
        else:
            chance = 50
        self.inputChance.ChangeValue(str(chance))
        for row in range(0, len(self.results)):
            roll = random.randint(1, 100)
            result = self.GetResult(chance, roll)
            self.rolls[row].ChangeValue(str(roll))
            self.results[row].ChangeValue(result)
    def GetResult(self, chance, roll):
        if chance < 0:
            roll -= chance
            chance = 0
        elif chance > 100:
            roll = roll - (chance - 100)
            chance = 100
        if chance == 100 and roll < 2:
            result = 'Critical4'
        elif self.Critical3[chance] >= roll:
            result = 'Critical3'
        elif self.Critical2[chance] >= roll:
            result = 'Critical2'
        elif self.Critical[chance] >= roll:
            result = 'Critical'
        elif self.Special[chance] >= roll:
            result = 'Special'
        elif chance >= roll:
            result = 'Normal'
        elif self.Miss[chance] >= roll:
            result = 'Miss'
        elif self.Failure[chance] >= roll:
            result = 'Failure'
        elif self.Fumble[chance] >= roll:
            result = 'Fumble'
        elif self.Fumble2[chance] >= roll:
            result = 'Fumble2'
        elif chance == 0 and roll >= 100:
            result = 'Fumble4'
        else:
            result = 'Fumble3'
        return result

class MainWindow(wx.Frame):
    """ Dice Roller's main window is simply a subclass of Frame """
    def __init__(self, parent, id, title):
        wx.Frame.__init__(self, parent, id, title, size=(-1,-1), style=wx.DEFAULT_FRAME_STYLE|wx.TAB_TRAVERSAL)
        self.DiceSets = []
        self.SkillSets = []
        ## Layout
        #   Number     Dice     Result
        # [        ] [       ] [       ]
        # [Roll All] [Add Set] [Del Set]
        self.sizerMain = wx.FlexGridSizer()
        self.sizerMain.SetFlexibleDirection(wx.BOTH)
        self.panelDice = wx.Panel(self)
        self.sizerMain.Add(self.panelDice, flag=wx.EXPAND)
        self.sizerDice = wx.FlexGridSizer(cols=3)
        self.sizerDice.SetFlexibleDirection(wx.BOTH)
        headNumber = wx.StaticText(self.panelDice, label='Number', style=wx.ALIGN_CENTER | wx.ST_NO_AUTORESIZE)
        headDice = wx.StaticText(self.panelDice, label='Dice', style=wx.ALIGN_CENTER | wx.ST_NO_AUTORESIZE)
        headResult = wx.StaticText(self.panelDice, label='Result', style=wx.ALIGN_CENTER | wx.ST_NO_AUTORESIZE)
        self.buttonRoll = wx.Button(self.panelDice, label='Roll All')
        self.buttonAddSet = wx.Button(self.panelDice, label='Add Set')
        self.buttonDelSet = wx.Button(self.panelDice, label='Del Set')
        self.sizerDice.Add(headNumber)
        self.sizerDice.Add(headDice)
        self.sizerDice.Add(headResult)
        self.sizerDice.Add(self.buttonRoll)
        self.sizerDice.Add(self.buttonAddSet)
        self.sizerDice.Add(self.buttonDelSet)
        self.panelDice.SetSizer(self.sizerDice)
        self.panelDice.SetAutoLayout(True)
        self.AddDiceSet(False)
        #
        self.panelSkills = wx.Panel(self)
        self.sizerMain.Add(self.panelSkills)
        self.sizerSkills = wx.FlexGridSizer(cols=7)
        self.sizerDice.SetFlexibleDirection(wx.BOTH)
        headNumSkills = wx.StaticText(self.panelSkills, label='Number', style=wx.ALIGN_CENTER | wx.ST_NO_AUTORESIZE)
        headScore = wx.StaticText(self.panelSkills, label='Score', style=wx.ALIGN_CENTER | wx.ST_NO_AUTORESIZE)
        headDifficulty = wx.StaticText(self.panelSkills, label='Difficulty', style=wx.ALIGN_CENTER | wx.ST_NO_AUTORESIZE)
        headAdj = wx.StaticText(self.panelSkills, label='adj', style=wx.ALIGN_CENTER | wx.ST_NO_AUTORESIZE)
        headChance = wx.StaticText(self.panelSkills, label='Chance', style=wx.ALIGN_CENTER | wx.ST_NO_AUTORESIZE)
        headRoll = wx.StaticText(self.panelSkills, label='Roll', style=wx.ALIGN_CENTER | wx.ST_NO_AUTORESIZE)
        headSkillResult = wx.StaticText(self.panelSkills, label='Result', style=wx.ALIGN_CENTER | wx.ST_NO_AUTORESIZE)
        self.buttonRollSkills = wx.Button(self.panelSkills, label='Roll All')
        self.buttonAddSkillsSet = wx.Button(self.panelSkills, label='Add Set')
        self.buttonDelSkillsSet = wx.Button(self.panelSkills, label='Del Set')
        self.sizerSkills.Add(headNumSkills)
        self.sizerSkills.Add(headScore)
        self.sizerSkills.Add(headDifficulty)
        self.sizerSkills.Add(headAdj)
        self.sizerSkills.Add(headChance)
        self.sizerSkills.Add(headRoll)
        self.sizerSkills.Add(headSkillResult)
        self.sizerSkills.Add(self.buttonRollSkills)
        self.sizerSkills.Add(self.buttonAddSkillsSet)
        self.sizerSkills.Add(self.buttonDelSkillsSet)
        self.panelSkills.SetSizer(self.sizerSkills)
        self.panelSkills.SetAutoLayout(True)
        self.AddSkillsSet(False)
        #
        self.SetSizer(self.sizerMain)
        self.SetAutoLayout(True)
        self.CreateStatusBar() # A Statusbar in the bottom of the window
        # Setting up the menu.
        filemenu= wx.Menu()
        filemenu.Append(ID_ABOUT, "&About"," Information about this program")
        filemenu.AppendSeparator()
        filemenu.Append(ID_EXIT,"E&xit"," Terminate the program")
        # Creating the menubar.
        menuBar = wx.MenuBar()
        menuBar.Append(filemenu,"&File") # Adding the "filemenu" to the MenuBar
        self.SetMenuBar(menuBar)  # Adding the MenuBar to the Frame content.
        wx.EVT_MENU(self, ID_ABOUT, self.OnAbout) # attach the menu-event ID_ABOUT to the
                                                  # method self.OnAbout
        wx.EVT_MENU(self, ID_EXIT, self.OnExit)   # attach the menu-event ID_EXIT to the
                                                  # method self.OnExit
        # Events
        self.buttonRoll.Bind(wx.EVT_BUTTON, self.RollAll)
        self.buttonAddSet.Bind(wx.EVT_BUTTON, self.AddDiceSet)
        self.buttonDelSet.Bind(wx.EVT_BUTTON, self.DelDiceSet)
        self.buttonRollSkills.Bind(wx.EVT_BUTTON, self.RollAllSkills)
        self.buttonAddSkillsSet.Bind(wx.EVT_BUTTON, self.AddSkillsSet)
        self.buttonDelSkillsSet.Bind(wx.EVT_BUTTON, self.DelSkillsSet)
        #
        self.Fit()
        self.Show(True)
    def RollAll(self, event):
        for diceSet in self.DiceSets:
            diceSet.UpdateSet(event)
    def AddDiceSet(self, event):
        diceSet = DiceSet(self.panelDice, 3 * len(self.DiceSets) + 3, 1)
        self.DiceSets.append(diceSet)
        self.Layout()
    def DelDiceSet(self, event):
        if (len(self.DiceSets) > 1):
          diceSet = self.DiceSets.pop()
          diceSet.Destroy()
          self.Layout()
    def RollAllSkills(self, event):
        for skillSet in self.SkillSets:
            skillSet.UpdateSet(event)
    def AddSkillsSet(self, event):
        skillSet = SkillSet(self.panelSkills, 7 * len(self.SkillSets) + 7, 1)
        self.SkillSets.append(skillSet)
        self.Layout()
    def DelSkillsSet(self, event):
        if (len(self.SkillSets) > 1):
          skillSet = self.SkillSets.pop()
          skillSet.Destroy()
          self.Layout()
    def OnAbout(self,e):
        d= wx.MessageDialog( self, " A dice rolling utility for Rune Master "
                            "by Tim Doty. For updates see "
                            "http://www.morfarpottery.com/RuneMaster/utilities.php",
                            "About Dice Roller", wx.OK)
                            # Create a message dialog box
        d.ShowModal() # Shows it
        d.Destroy() # finally destroy it when finished.
    def OnExit(self,e):
        self.Close(True)  # Close the frame.

app = wx.App()
frame = MainWindow(None, wx.ID_ANY, "Die Roller")
app.MainLoop()

